/** \file nv_rtSoundItf.h 
 **
 **/ /***********************************************************************/
#ifndef __Sound_H__
#define __Sound_H__

#define SoundITF_VERSION "1.12"

#include "sapi_smartptr_itf.h"

using namespace sapi;

class ISound;
class ISound3D;
/*************************************************************************/ /**
 **
 ** must be implemented by the client if it wants to get back events
 ** 
 **/ /*************************************************************************/
class IEventsSound
{
public:
	virtual bool OutputString(ISound *, LPCSTR) { return false; };
	virtual bool OutputError(ISound *, LPCSTR) { return false; };
};
/**
 **	Interface for Sound system embedded into a rt module
 **/
class ISound
{
public:
	DECLSMARTREF(); ///< Macro for GetRef(), AddRef(), Release() , QueryInterface() implementations

	/// \name Sound interface methods
	//@{
	virtual unsigned int GetPos() = 0;/// get the current position
	virtual void Play(unsigned int pos = 0) = 0;/// play the sound at the given position
	virtual void Stop() = 0;/// stop the sound
	virtual void SetVolume(float) = 0;/// set the volume
	virtual void Loop(bool) = 0;/// set looping
	virtual bool IsPlaying() = 0;/// query the sound... is it playing?
	virtual bool IsValid() = 0;/// are we a valid sound?
	virtual void SetFrequency(int freq) = 0;/// can all API's do this?
	virtual bool  Looping () = 0;
	virtual float Volume  () = 0;
	//@}
	/// \name register to events from the occurence
	//@{
	virtual void Register(IEventsSound *pClient) = 0;
	virtual void UnRegister(IEventsSound *pClient) = 0;
	//@}
	virtual bool Init(LPCSTR filename, bool loop=true, float volume=1.0f, unsigned long windowhandle=0) = 0;
	virtual ISound3D *Init3D(LPCSTR filename, bool loop=true, float volume=1.0f, unsigned long windowhandle=0) = 0;
};

/**
 **	Interface for Sound system embedded into a rt module
 **/
class ISound3D : public ISound
{
public:
	DECLSMARTREF(); ///< Macro for GetRef(), AddRef(), Release() , QueryInterface() implementations

	/// \name 3D sound interface methods
	//@{
    virtual void SetDistanceAttenuation(bool da) =0;//{ m_distAtten = da; }
    virtual bool GetDistanceAttenuation() =0;//const  { return m_distAtten; }
  // Spatialization routines
    virtual void SetMinDistance(float dist)      = 0;
    virtual void SetMaxDistance(float dist)      = 0;
    virtual void SetUmbraAngle(float theta)      = 0;
    virtual void SetPenumbraAngle(float theta)   = 0;
    virtual void SetOuterVolume(float vol)       = 0;
    virtual void SetPosition(float x, float y, float z) = 0;
    virtual void SetVelocity(float vx, float vy, float vz) = 0;
    virtual void SetConeDirection(float x, float y, float z) = 0;
	//@}
	/// \name register to events from the occurence
	//@{
	virtual void Register(IEventsSound *pClient) = 0;
	virtual void UnRegister(IEventsSound *pClient) = 0;
	//@}
};

#endif
